/****************************************************************************
** Unit demonstrating the use of extended memory in Turbo C/C++ programs.  **
**  by Steven H Don                                                        **
**                                                                         **
** For questions, feel free to e-mail me.                                  **
**                                                                         **
**    shd@earthling.net                                                    **
**    http://shd.cjb.net                                                   **
**                                                                         **
****************************************************************************/

#include <stdio.h>
#include <conio.h>
#include <mem.h>
#include "xms.h"

//A pointer to the screen
unsigned char far *screen;
unsigned int XMSHandle;

void VGAScreen ()
/*This sets the display to VGA 320x200 in 256 colours*/
{
  asm {
    mov ax, 0x13
    int 0x10
  }
}

void TextScreen ()
/*This resets the display to text mode*/
{
  asm {
    mov ax, 0x3
    int 0x10
  }
}

void Gradient ()
{
  //This displays a smooth pattern on screen, just for demonstration purposes
  unsigned int OffSet;

  for (OffSet = 0; OffSet < 63999; OffSet++)
    screen [OffSet] = OffSet % 315;
}

void RedPalette ()
{
  //This sets up a smooth red palette

  unsigned int Counter;

  for (Counter = 0; Counter <= 255; Counter++)
  {
    //Set up a palette...
    outportb (0x3C8, Counter);
    //Increasingly bright red
    outportb (0x3C9, Counter >> 2);
    //No green
    outportb (0x3C9, 0);
    //No blue either
    outportb (0x3C9, 0);
  }
}

void main ()
{
  //Check whether XMS is actually there
  if (XMSInit ())
    printf ("XMS routines installed.\n");
  else
    printf ("No XMS found.\n");

  //Switch to VGA
  VGAScreen ();
  //Set up a pointer to the vga memory at A000:0000
  screen = (unsigned char far *)MK_FP (0xA000, 0);

  //Set up a smooth red palette
  RedPalette ();

  //Set up the gradient
  Gradient ();

  //Assign an extended memory block 64 K-bytes in size (65536 bytes)
  XMSGetMem (&XMSHandle, 64);

  /*Copy from handle 0 (conventional memory) at $A000:0000 (video memory)
  to XMS memory allocated above at offset 0 into the extended memory block
  for a total of 64000 bytes. Note that you can only copy an EVEN number
  of bytes. So a copy of 5123 bytes, for example, is not possible. */
  XMSCopy (0, 0xA0000000, XMSHandle, 0, 64000);

  //Wait for a keypress
  getch ();

  //Clear the screen
  _fmemset (screen, 0, 64000);

  //Wait for a keypress
  getch ();

  /*Copy the extended memory into the video memory - this brings back the
  gradient*/
  XMSCopy (XMSHandle, 0, 0, 0xA0000000, 64000);

  //Release the extended memory block
  XMSFreeMem (XMSHandle);

  //Wait for a keypress
  getch ();

  //Return to text
  TextScreen ();
}